@@ -1,4 +1,4 @@
-/* $PostgreSQL: pgsql/src/interfaces/ecpg/ecpglib/connect.c,v 1.24.4.5 2006/06/21 11:38:35 meskes Exp $ */
+/* $PostgreSQL: pgsql/src/interfaces/ecpg/ecpglib/connect.c,v 1.26.2.1 2005/11/30 12:50:37 meskes Exp $ */
 
 #define POSTGRES_ECPG_INTERNAL
 #include "postgres_fe.h"
@@ -16,7 +16,6 @@
 static pthread_mutex_t connections_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_key_t actual_connection_key;
 static pthread_once_t actual_connection_key_once = PTHREAD_ONCE_INIT;
-
 #endif
 static struct connection *actual_connection = NULL;
 static struct connection *all_connections = NULL;
@@ -38,10 +37,13 @@ ecpg_get_connection_nr(const char *connection_name)
 	{
 #ifdef ENABLE_THREAD_SAFETY
 		ret = pthread_getspecific(actual_connection_key);
-		/* if no connection in TSD for this thread, get the global default connection
-		 * and hope the user knows what they're doing (i.e. using their own mutex to
-		 * protect that connection from concurrent accesses */
-		if(NULL == ret)
+
+		/*
+		 * if no connection in TSD for this thread, get the global default
+		 * connection and hope the user knows what they're doing (i.e. using
+		 * their own mutex to protect that connection from concurrent accesses
+		 */
+		if (NULL == ret)
 		{
 			ECPGlog("no TSD connection, going for global\n");
 			ret = actual_connection;
@@ -76,13 +78,16 @@ ECPGget_connection(const char *connection_name)
 	{
 #ifdef ENABLE_THREAD_SAFETY
 		ret = pthread_getspecific(actual_connection_key);
-		/* if no connection in TSD for this thread, get the global default connection
-         * and hope the user knows what they're doing (i.e. using their own mutex to
-         * protect that connection from concurrent accesses */
-        if(NULL == ret)
+
+		/*
+		 * if no connection in TSD for this thread, get the global default
+		 * connection and hope the user knows what they're doing (i.e. using
+		 * their own mutex to protect that connection from concurrent accesses
+		 */
+		if (NULL == ret)
 		{
 			ECPGlog("no TSD connection here either, using global\n");
-            ret = actual_connection;
+			ret = actual_connection;
 		}
 		else
 			ECPGlog("got TSD connection\n");
@@ -222,9 +227,6 @@ ECPGnoticeReceiver(void *arg, const PGresult *result)
 	if (sqlstate == NULL)
 		sqlstate = ECPG_SQLSTATE_ECPG_INTERNAL_ERROR;
 
-	if (message == NULL)  /* Shouldn't happen, but need to be sure */
-		message = "No message received";
-
 	/* these are not warnings */
 	if (strncmp(sqlstate, "00", 2) == 0)
 		return;
@@ -278,8 +280,8 @@ ECPGconnect(int lineno, int c, const char *name, const char *user, const char *p
 
 		/*
 		 * Informix uses an environment variable DBPATH that overrides the
-		 * connection parameters given here. We do the same with PG_DBPATH
-		 * as the syntax is different.
+		 * connection parameters given here. We do the same with PG_DBPATH as
+		 * the syntax is different.
 		 */
 		envname = getenv("PG_DBPATH");
 		if (envname)
@@ -297,20 +299,20 @@ ECPGconnect(int lineno, int c, const char *name, const char *user, const char *p
 		connection_name = "DEFAULT";
 
 	if (dbname != NULL)
-	{	
+	{
 		/* get the detail information out of dbname */
 		if (strchr(dbname, '@') != NULL)
 		{
 			/* old style: dbname[@server][:port] */
 			tmp = strrchr(dbname, ':');
-			if (tmp != NULL)		/* port number given */
+			if (tmp != NULL)	/* port number given */
 			{
 				port = ECPGstrdup(tmp + 1, lineno);
 				*tmp = '\0';
 			}
 
 			tmp = strrchr(dbname, '@');
-			if (tmp != NULL)		/* host name given */
+			if (tmp != NULL)	/* host name given */
 			{
 				host = ECPGstrdup(tmp + 1, lineno);
 				*tmp = '\0';
@@ -401,7 +403,6 @@ ECPGconnect(int lineno, int c, const char *name, const char *user, const char *p
 							ECPGfree(realname);
 						if (dbname)
 							ECPGfree(dbname);
-						ecpg_finish(this);
 						return false;
 					}
 				}
@@ -455,6 +456,10 @@ ECPGconnect(int lineno, int c, const char *name, const char *user, const char *p
 		const char *errmsg = PQerrorMessage(this->connection);
 		char	   *db = realname ? realname : "<DEFAULT>";
 
+		ecpg_finish(this);
+#ifdef ENABLE_THREAD_SAFETY
+		pthread_mutex_unlock(&connections_mutex);
+#endif
 		ECPGlog("connect: could not open database %s on %s port %s %s%s%s%s in line %d\n\t%s\n",
 				db,
 				host ? host : "<DEFAULT>",
@@ -463,11 +468,6 @@ ECPGconnect(int lineno, int c, const char *name, const char *user, const char *p
 				user ? "for user " : "", user ? user : "",
 				lineno, errmsg);
 
-		ecpg_finish(this);
-#ifdef ENABLE_THREAD_SAFETY
-		pthread_mutex_unlock(&connections_mutex);
-#endif
-
 		ECPGraise(lineno, ECPG_CONNECT, ECPG_SQLSTATE_SQLCLIENT_UNABLE_TO_ESTABLISH_SQLCONNECTION, db);
 		if (host)
 			ECPGfree(host);
